"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _BaseTexture = _interopRequireDefault(require("./BaseTexture"));

var _settings = _interopRequireDefault(require("../settings"));

var _TextureUvs = _interopRequireDefault(require("./TextureUvs"));

var _eventemitter = _interopRequireDefault(require("eventemitter3"));

var _math = require("../math");

var _VideoBaseTexture = _interopRequireDefault(require("./VideoBaseTexture"));

var _utils = require("../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Texture extends _eventemitter.default {
  constructor(baseTexture, frame, orig, trim, rotate, anchor) {
    super();
    this.noFrame = false;

    if (!frame) {
      this.noFrame = true;
      frame = new _math.Rectangle(0, 0, 1, 1);
    }

    this._frame = frame;
    this.trim = trim;
    this.valid = false;
    this.requiresUpdate = false;
    this._uvs = null;
    this.orig = orig || frame;
    this._rotate = Number(rotate || 0);

    if (rotate === true) {
      this._rotate = 2;
    } else if (this._rotate % 2 !== 0) {
      throw new Error("attempt to use diamond-shaped UVs.");
    }

    this.initBaseTexture(baseTexture, frame);
    this.defaultAnchor = anchor ? new _math.Point(anchor.x, anchor.y) : new _math.Point(0, 0);
    this._updateID = 0;
    this.transform = null;
    this.textureCacheIds = [];
  }

  update() {
    this.baseTexture.update();
  }

  updateSource(imageUrl) {
    this.baseTexture.updateSource(imageUrl);
  }

  initBaseTexture(baseTexture, frame) {
    if (baseTexture instanceof Texture) baseTexture = baseTexture.baseTexture;
    this.baseTexture = baseTexture;

    if (baseTexture.hasLoaded) {
      if (this.noFrame) {
        frame = new _math.Rectangle(0, 0, baseTexture.width, baseTexture.height);
        baseTexture.on("update", this.onBaseTextureUpdated, this);
      }

      this.frame = frame;
      baseTexture.adaptedTexture();
    } else {
      baseTexture.once("loaded", this.onBaseTextureLoaded, this);
    }

    baseTexture.on("error", this.onBaseTextureError, this);
  }

  onBaseTextureLoaded(baseTexture) {
    this._updateID++;

    if (this.noFrame) {
      this.frame = new _math.Rectangle(0, 0, baseTexture.width, baseTexture.height);
    } else {
      this.frame = this._frame;
    }

    baseTexture.adaptedTexture();
    this.baseTexture.on("update", this.onBaseTextureUpdated, this);
    this.emit("update", this);
  }

  onBaseTextureError(e) {
    this.emit("error", e);
  }

  onBaseTextureUpdated(baseTexture) {
    this._updateID++;
    this._frame.width = baseTexture.width;
    this._frame.height = baseTexture.height;
    baseTexture.adaptedTexture();
    this.emit("update", this);
  }

  destroy(destroyBase) {
    if (this.baseTexture) {
      if (destroyBase) {
        if (_utils.TextureCache[this.baseTexture.imageUrl]) {
          Texture.removeFromCache(this.baseTexture.imageUrl);
        }

        this.baseTexture.destroy();
      }

      this.baseTexture.off("update", this.onBaseTextureUpdated, this);
      this.baseTexture.off("loaded", this.onBaseTextureLoaded, this);
      this.baseTexture.off("error", this.onBaseTextureError, this);
      this.baseTexture = null;
    }

    this._frame = null;
    this._uvs = null;
    this.trim = null;
    this.orig = null;
    this.valid = false;
    Texture.removeFromCache(this);
    this.textureCacheIds = null;
  }

  clone() {
    return new Texture(this.baseTexture, this.frame, this.orig, this.trim, this.rotate, this.defaultAnchor);
  }

  _updateUvs() {
    if (!this._uvs) this._uvs = new _TextureUvs.default();

    this._uvs.set(this._frame, this.baseTexture, this.rotate);

    this._updateID++;
  }

  static fromImage(imageUrl, crossorigin, scaleMode, sourceScale) {
    var texture = _utils.TextureCache[imageUrl];
    if (texture) return texture;

    if (crossorigin instanceof _math.Rectangle) {
      texture = new Texture(_BaseTexture.default.fromImage(imageUrl), crossorigin, scaleMode, sourceScale);
    } else {
      texture = new Texture(_BaseTexture.default.fromImage(imageUrl, crossorigin, scaleMode, sourceScale));
    }

    Texture.addToCache(texture, imageUrl);
    return texture;
  }

  static fromFrame(frameId) {
    var texture = _utils.TextureCache[frameId];

    if (!texture) {
      throw new Error("The frameId \"" + frameId + "\" does not exist in cache");
    }

    return texture;
  }

  static fromCanvas(canvas, scaleMode, origin) {
    if (origin === void 0) {
      origin = "canvas";
    }

    return new Texture(_BaseTexture.default.fromCanvas(canvas, scaleMode, origin));
  }

  static fromVideo(video, scaleMode, crossorigin, autoPlay) {
    if (typeof video === "string") {
      return Texture.fromVideoUrl(video, scaleMode, crossorigin, autoPlay);
    }

    return new Texture(_VideoBaseTexture.default.fromVideo(video, scaleMode, autoPlay));
  }

  static fromVideoUrl(videoUrl, scaleMode, crossorigin, autoPlay) {
    return new Texture(_VideoBaseTexture.default.fromUrl(videoUrl, scaleMode, crossorigin, autoPlay));
  }

  static from(source) {
    if (typeof source === "string") {
      var texture = _utils.TextureCache[source];

      if (!texture) {
        var isVideo = source.match(/\.(mp4|webm|ogg|h264|avi|mov)$/) !== null;
        if (isVideo) return Texture.fromVideoUrl(source);
        return Texture.fromImage(source);
      }

      return texture;
    } else if (source instanceof HTMLImageElement) {
      return new Texture(_BaseTexture.default.from(source));
    } else if (source instanceof HTMLCanvasElement) {
      return Texture.fromCanvas(source, _settings.default.SCALE_MODE, "HTMLCanvasElement");
    } else if (source instanceof HTMLVideoElement) {
      return Texture.fromVideo(source);
    } else if (source instanceof _BaseTexture.default) {
      return new Texture(source);
    }

    return source;
  }

  static fromLoader(source, imageUrl, name) {
    var baseTexture = new _BaseTexture.default(source, undefined, (0, _utils.getResolutionOfUrl)(imageUrl));
    var texture = new Texture(baseTexture);
    baseTexture.imageUrl = imageUrl;
    if (!name) name = imageUrl;

    _BaseTexture.default.addToCache(texture.baseTexture, name);

    Texture.addToCache(texture, name);

    if (name !== imageUrl) {
      _BaseTexture.default.addToCache(texture.baseTexture, imageUrl);

      Texture.addToCache(texture, imageUrl);
    }

    return texture;
  }

  static addToCache(texture, id) {
    if (!id) return;
    if (texture.textureCacheIds.indexOf(id) === -1) texture.textureCacheIds.push(id);
    if (_utils.TextureCache[id]) console.warn("Texture  id [" + id + "] that already added");
    _utils.TextureCache[id] = texture;
  }

  static removeFromCache(texture) {
    if (typeof texture === "string") {
      var textureFromCache = _utils.TextureCache[texture];

      if (textureFromCache) {
        var index = textureFromCache.textureCacheIds.indexOf(texture);
        if (index > -1) textureFromCache.textureCacheIds.splice(index, 1);
        delete _utils.TextureCache[texture];
        return textureFromCache;
      }
    } else if (texture && texture.textureCacheIds) {
      for (var i = 0; i < texture.textureCacheIds.length; ++i) {
        if (_utils.TextureCache[texture.textureCacheIds[i]] === texture) {
          delete _utils.TextureCache[texture.textureCacheIds[i]];
        }
      }

      texture.textureCacheIds.length = 0;
      return texture;
    }

    return null;
  }

  get frame() {
    return this._frame;
  }

  set frame(frame) {
    this._frame = frame;
    this.noFrame = false;
    var {
      x,
      y,
      width,
      height
    } = frame;
    var xNotFit = x + width > this.baseTexture.width;
    var yNotFit = y + height > this.baseTexture.height;

    if (xNotFit || yNotFit) {
      var relationship = xNotFit && yNotFit ? "and" : "or";
      var errorX = "X: " + x + " + " + width + " = " + (x + width) + " > " + this.baseTexture.width;
      var errorY = "Y: " + y + " + " + height + " = " + (y + height) + " > " + this.baseTexture.height;
      throw new Error("Texture Error: frame does not fit inside the base Texture dimensions: " + (errorX + " " + relationship + " " + errorY));
    }

    this.valid = width && height && this.baseTexture.hasLoaded;

    if (!this.trim && !this.rotate) {
      this.orig = frame;
    }

    if (this.valid) {
      this._updateUvs();
    }
  }

  get rotate() {
    return this._rotate;
  }

  set rotate(rotate) {
    this._rotate = rotate;

    if (this.valid) {
      this._updateUvs();
    }
  }

  get width() {
    return this.orig.width;
  }

  get height() {
    return this.orig.height;
  }

}

exports.default = Texture;

function removeAllHandlers(tex) {
  tex.destroy = function _emptyDestroy() {
    /* empty */
  };

  tex.on = function _emptyOn() {
    /* empty */
  };

  tex.once = function _emptyOnce() {
    /* empty */
  };

  tex.emit = function _emptyEmit() {
    /* empty */
  };
}

Texture.EMPTY = new Texture(new _BaseTexture.default());
removeAllHandlers(Texture.EMPTY);
removeAllHandlers(Texture.EMPTY.baseTexture);
//# sourceMappingURL=Texture.js.map